package mcmultipart.block;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import mcmultipart.MCMultiPartMod;
import mcmultipart.client.multipart.ICustomHighlightPart;
import mcmultipart.client.multipart.IHitEffectsPart;
import mcmultipart.client.multipart.IHitEffectsPart.AdvancedEffectRenderer;
import mcmultipart.client.multipart.ISmartMultipartModel;
import mcmultipart.client.multipart.MultipartStateMapper;
import mcmultipart.microblock.IMicroblockTile;
import mcmultipart.microblock.MicroblockContainer;
import mcmultipart.multipart.IMultipart;
import mcmultipart.multipart.Multipart;
import mcmultipart.multipart.MultipartContainer;
import mcmultipart.multipart.MultipartRegistry;
import mcmultipart.multipart.PartState;
import mcmultipart.raytrace.PartMOP;
import mcmultipart.raytrace.RayTraceUtils;
import mcmultipart.raytrace.RayTraceUtils.RayTraceResultPart;
import net.minecraft.block.Block;
import net.minecraft.block.BlockContainer;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.particle.EffectRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.resources.model.IBakedModel;
import net.minecraft.client.resources.model.ModelResourceLocation;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumWorldBlockLayer;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.client.event.DrawBlockHighlightEvent;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.property.ExtendedBlockState;
import net.minecraftforge.common.property.IExtendedBlockState;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

/**
 * An implementation of {@link BlockContainer} that allows your block to act as a microblock container.<br/>
 * Extend this class if you want your block to be able to have microblocks, but not any other kinds of multiparts. Otherwise, extend
 * {@link Multipart} or make a custom implementation of {@link IMultipart}.<br/>
 * All the overriden methods have a "default" counterpart that allows you to handle interactions with your block if the player isn't
 * interacting with a multipart.<br/>
 * Extend {@link TileCoverable} or implement {@link IMicroblockTile} and return it in {@link BlockCoverable#createNewTileEntity(World, int)}
 * if you want a custom tile entity for your block.
 */
public class BlockCoverable extends BlockContainer {

    public BlockCoverable(Material material) {

        super(material);
        MinecraftForge.EVENT_BUS.register(this);
    }

    @Override
    public TileEntity func_149915_a(World worldIn, int meta) {

        return new TileCoverable();
    }

    private IMicroblockTile getMicroblockTile(IBlockAccess world, BlockPos pos) {

        TileEntity tile = world.func_175625_s(pos);
        return tile instanceof IMicroblockTile ? (IMicroblockTile) tile : null;
    }

    @Override
    public final MovingObjectPosition func_180636_a(World world, BlockPos pos, Vec3 start, Vec3 end) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        RayTraceResultPart result = tile != null ? tile.getMicroblockContainer().getPartContainer().collisionRayTrace(start, end) : null;
        MovingObjectPosition hit = collisionRayTraceDefault(world, pos, start, end);
        if (result == null) return hit;
        if (hit != null && hit.field_72307_f.func_72436_e(start) < result.squareDistanceTo(start)) return hit;
        result.setBounds(world, pos);
        return result.hit;
    }

    private boolean rayTracingDefault = false;

    public MovingObjectPosition collisionRayTraceDefault(World world, BlockPos pos, Vec3 start, Vec3 end) {

        rayTracingDefault = true;
        MovingObjectPosition hit = super.func_180636_a(world, pos, start, end);
        rayTracingDefault = false;
        return hit;
    }

    @Override
    public final void func_180654_a(IBlockAccess world, BlockPos pos) {

        if (rayTracingDefault) setBlockBoundsBasedOnStateDefault(world, pos);
    }

    public void setBlockBoundsBasedOnStateDefault(IBlockAccess world, BlockPos pos) {

    }

    @Override
    public final void func_180638_a(World world, BlockPos pos, IBlockState state, AxisAlignedBB mask, List<AxisAlignedBB> list,
            Entity collidingEntity) {

        addCollisionBoxesToListDefault(world, pos, state, mask, list, collidingEntity);
        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile != null) tile.getMicroblockContainer().getPartContainer().addCollisionBoxes(mask, list, collidingEntity);
    }

    public void addCollisionBoxesToListDefault(World worldIn, BlockPos pos, IBlockState state, AxisAlignedBB mask,
            List<AxisAlignedBB> list, Entity collidingEntity) {

        super.func_180638_a(worldIn, pos, state, mask, list, collidingEntity);
    }

    @Override
    public final int getLightValue(IBlockAccess world, BlockPos pos) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        return Math.max(tile != null ? tile.getMicroblockContainer().getPartContainer().getLightValue() : 0,
                getLightValueDefault(world, pos));
    }

    public int getLightValueDefault(IBlockAccess world, BlockPos pos) {

        return super.getLightValue(world, pos);
    }

    @Override
    public final ItemStack getPickBlock(MovingObjectPosition target, World world, BlockPos pos, EntityPlayer player) {

        if (target instanceof PartMOP) {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            return tile != null ? tile.getMicroblockContainer().getPartContainer().getPickBlock(player, (PartMOP) target) : null;
        }
        return getPickBlockDefault(target, world, pos, player);
    }

    public ItemStack getPickBlockDefault(MovingObjectPosition target, World world, BlockPos pos, EntityPlayer player) {

        return super.getPickBlock(target, world, pos, player);
    }

    private IMicroblockTile brokenTile = null;
    private boolean harvestingWrapper = false;

    @Override
    public final void func_180657_a(World world, EntityPlayer player, BlockPos pos, IBlockState state, TileEntity te) {

        brokenTile = te instanceof IMicroblockTile ? (IMicroblockTile) te : null;
        defaultHarvestBlock(world, player, pos, state, te);
        brokenTile = null;
    }

    public void defaultHarvestBlock(World worldIn, EntityPlayer player, BlockPos pos, IBlockState state, TileEntity te) {

        super.func_180657_a(worldIn, player, pos, state, te);
    }

    @Override
    public List<ItemStack> getDrops(IBlockAccess world, BlockPos pos, IBlockState state, int fortune) {

        if (harvestingWrapper) return getDropsDefault(world, pos, state, fortune);
        IMicroblockTile brokenTile = getMicroblockTile(world, pos);
        if (brokenTile == null) brokenTile = this.brokenTile;
        if (brokenTile == null) return getDropsDefault(world, pos, state, fortune);
        List<ItemStack> drops = new ArrayList<ItemStack>();
        drops.addAll(getDropsDefault(world, pos, state, fortune));
        drops.addAll(brokenTile.getMicroblockContainer().getPartContainer().getDrops());
        return drops;
    }

    public List<ItemStack> getDropsDefault(IBlockAccess world, BlockPos pos, IBlockState state, int fortune) {

        return super.getDrops(world, pos, state, fortune);
    }

    @Override
    public final boolean removedByPlayer(World world, BlockPos pos, EntityPlayer player, boolean willHarvest) {

        MovingObjectPosition hit = reTraceAll(world, pos, player);
        if (hit instanceof PartMOP) {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            if (tile != null) tile.getMicroblockContainer().getPartContainer().harvest(player, (PartMOP) hit);
            return false;
        } else {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            MultipartContainer container = tile != null ? tile.getMicroblockContainer().getPartContainer() : null;
            if (container.getParts().isEmpty()) {
                return removedByPlayerDefault(world, pos, player, willHarvest);
            } else {
                if (!removedByPlayerDefault(world, pos, player, willHarvest)) return false;
                world.func_175713_t(pos);
                if (!world.func_180501_a(pos, MCMultiPartMod.multipart.func_176223_P(), 0)) return false;
                world.func_175713_t(pos);
                world.func_175690_a(pos, new TileMultipart(container));
                world.func_175689_h(pos);
                harvestingWrapper = true;
                func_180657_a(world, player, pos, world.func_180495_p(pos), world.func_175625_s(pos));
                harvestingWrapper = false;
                return false;
            }
        }
    }

    public boolean removedByPlayerDefault(World world, BlockPos pos, EntityPlayer player, boolean willHarvest) {

        return super.removedByPlayer(world, pos, player, willHarvest);
    }

    @Override
    public final float func_180647_a(EntityPlayer player, World world, BlockPos pos) {

        MovingObjectPosition hit = reTraceAll(world, pos, player);
        if (hit instanceof PartMOP) {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            return tile != null ? tile.getMicroblockContainer().getPartContainer().getHardness(player, (PartMOP) hit) : 0F;
        } else {
            return getPlayerRelativeBlockHardnessDefault(player, world, pos);
        }
    }

    public float getPlayerRelativeBlockHardnessDefault(EntityPlayer player, World world, BlockPos pos) {

        return super.func_180647_a(player, world, pos);
    }

    @Override
    public final boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumFacing side, float hitX,
            float hitY, float hitZ) {

        MovingObjectPosition hit = reTraceAll(world, pos, player);
        if (hit instanceof PartMOP) {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            return tile != null ? tile.getMicroblockContainer().getPartContainer()
                    .onActivated(player, player.func_71045_bC(), (PartMOP) hit) : false;
        } else {
            return onBlockActivatedDefault(world, pos, state, player, side, hitX, hitY, hitZ);
        }
    }

    public boolean onBlockActivatedDefault(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumFacing side, float hitX,
            float hitY, float hitZ) {

        return super.func_180639_a(world, pos, state, player, side, hitX, hitY, hitZ);
    }

    @Override
    public final void func_180649_a(World world, BlockPos pos, EntityPlayer player) {

        MovingObjectPosition hit = reTraceAll(world, pos, player);
        if (hit instanceof PartMOP) {
            IMicroblockTile tile = getMicroblockTile(world, pos);
            if (tile != null)
                tile.getMicroblockContainer().getPartContainer().onClicked(player, player.func_71045_bC(), (PartMOP) hit);
        } else {
            onBlockClickedDefault(world, pos, player);
        }
    }

    public void onBlockClickedDefault(World world, BlockPos pos, EntityPlayer player) {

        super.func_180649_a(world, pos, player);
    }

    @Override
    public final void func_176204_a(World world, BlockPos pos, IBlockState state, Block neighborBlock) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile != null) tile.getMicroblockContainer().getPartContainer().onNeighborBlockChange(neighborBlock);
        onNeighborBlockChangeDefault(world, pos, state, neighborBlock);
    }

    public void onNeighborBlockChangeDefault(World world, BlockPos pos, IBlockState state, Block neighborBlock) {

    }

    @Override
    public final void onNeighborChange(IBlockAccess world, BlockPos pos, BlockPos neighbor) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile != null)
            tile.getMicroblockContainer()
                    .getPartContainer()
                    .onNeighborTileChange(
                            EnumFacing.func_176737_a(neighbor.func_177958_n() - pos.func_177958_n(), neighbor.func_177956_o() - pos.func_177956_o(), neighbor.func_177952_p()
                                    - pos.func_177952_p()));
        onNeighborChangeDefault(world, pos, neighbor);
    }

    public void onNeighborChangeDefault(IBlockAccess world, BlockPos pos, BlockPos neighbor) {

    }

    @Override
    public final void func_176199_a(World world, BlockPos pos, Entity entity) {

        onEntityCollidedWithBlockDefault(world, pos, entity);
        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile == null) return;
        tile.getMicroblockContainer().getPartContainer().onEntityStanding(entity);
    }

    public void onEntityCollidedWithBlockDefault(World worldIn, BlockPos pos, Entity entityIn) {

        super.func_176199_a(worldIn, pos, entityIn);
    }

    @Override
    public final void func_180634_a(World world, BlockPos pos, IBlockState state, Entity entity) {

        onEntityCollidedWithBlockDefault(world, pos, state, entity);
        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile == null) return;
        tile.getMicroblockContainer().getPartContainer().onEntityCollided(entity);
    }

    public void onEntityCollidedWithBlockDefault(World worldIn, BlockPos pos, IBlockState state, Entity entityIn) {

        super.func_180634_a(worldIn, pos, state, entityIn);
    }

    @Override
    public Boolean isAABBInsideMaterial(World world, BlockPos pos, AxisAlignedBB aabb, Material material) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        Boolean def = isAABBInsideMaterialDefault(world, pos, aabb, material);
        if (tile == null) return def;
        Boolean is = tile.getMicroblockContainer().getPartContainer().isAABBInsideMaterial(aabb, material);
        if ((def != null && def == true) || (is != null && is == true)) return true;
        if ((def != null && def == false) || (is != null && is == false)) return true;
        return null;
    }

    public Boolean isAABBInsideMaterialDefault(World world, BlockPos pos, AxisAlignedBB boundingBox, Material materialIn) {

        return super.isAABBInsideMaterial(world, pos, boundingBox, materialIn);
    }

    @Override
    public Boolean isEntityInsideMaterial(World world, BlockPos pos, IBlockState state, Entity entity, double yToTest, Material material,
            boolean testingHead) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        Boolean def = isEntityInsideMaterialDefault(world, pos, state, entity, yToTest, material, testingHead);
        if (tile == null) return def;
        Boolean is = tile.getMicroblockContainer().getPartContainer().isEntityInsideMaterial(entity, yToTest, material, testingHead);
        if ((def != null && def == true) || (is != null && is == true)) return true;
        if ((def != null && def == false) || (is != null && is == false)) return true;
        return null;
    }

    public Boolean isEntityInsideMaterialDefault(World world, BlockPos blockpos, IBlockState iblockstate, Entity entity, double yToTest,
            Material materialIn, boolean testingHead) {

        return super.isEntityInsideMaterial(world, blockpos, iblockstate, entity, yToTest, materialIn, testingHead);
    }

    @Override
    public boolean func_149744_f() {

        return true;
    }

    @Override
    public final boolean canConnectRedstone(IBlockAccess world, BlockPos pos, EnumFacing side) {

        if (side == null) return false;
        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container.getPartContainer().canConnectRedstone(side)) return true;
        return canConnectRedstoneDefault(world, pos, side, container);
    }

    public boolean canConnectRedstoneDefault(IBlockAccess world, BlockPos pos, EnumFacing side, MicroblockContainer partContainer) {

        return false;
    }

    @Override
    public final int func_180656_a(IBlockAccess world, BlockPos pos, IBlockState state, EnumFacing side) {

        if (side == null) return 0;
        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container == null) return getWeakPowerDefault(world, pos, state, side, null);
        return Math.max(container.getPartContainer().getWeakSignal(side), getWeakPowerDefault(world, pos, state, side, container));
    }

    public int getWeakPowerDefault(IBlockAccess world, BlockPos pos, IBlockState state, EnumFacing side, MicroblockContainer partContainer) {

        return super.func_180656_a(world, pos, state, side);
    }

    @Override
    public final int func_176211_b(IBlockAccess world, BlockPos pos, IBlockState state, EnumFacing side) {

        if (side == null) return 0;
        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container == null) return getStrongPowerDefault(world, pos, state, side, null);
        return Math.max(container.getPartContainer().getStrongSignal(side), getStrongPowerDefault(world, pos, state, side, container));
    }

    public int getStrongPowerDefault(IBlockAccess world, BlockPos pos, IBlockState state, EnumFacing side, MicroblockContainer partContainer) {

        return super.func_176211_b(world, pos, state, side);
    }

    @Override
    public final boolean isSideSolid(IBlockAccess world, BlockPos pos, EnumFacing side) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container == null) return false;
        return container.getPartContainer().isSideSolid(side) || isSideSolidDefault(world, pos, side);
    }

    public boolean isSideSolidDefault(IBlockAccess world, BlockPos pos, EnumFacing side) {

        return super.isSideSolid(world, pos, side);
    }

    @Override
    public final boolean canPlaceTorchOnTop(IBlockAccess world, BlockPos pos) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container == null) return false;
        return container.getPartContainer().canPlaceTorchOnTop() || canPlaceTorchOnTopDefault(world, pos);
    }

    public boolean canPlaceTorchOnTopDefault(IBlockAccess world, BlockPos pos) {

        return super.canPlaceTorchOnTop(world, pos);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(World world, BlockPos pos, IBlockState state, Random rand) {

        randomDisplayTickDefault(world, pos, state, rand);
        IMicroblockTile tile = getMicroblockTile(world, pos);
        MicroblockContainer container = tile != null ? tile.getMicroblockContainer() : null;
        if (container != null) container.getPartContainer().randomDisplayTick(rand);
    }

    @SideOnly(Side.CLIENT)
    public void randomDisplayTickDefault(World world, BlockPos pos, IBlockState state, Random rand) {

        super.func_180655_c(world, pos, state, rand);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public final boolean addDestroyEffects(World world, BlockPos pos, EffectRenderer effectRenderer) {

        PartMOP hit = reTrace(world, pos, MCMultiPartMod.proxy.getPlayer());
        if (hit != null) {
            if (hit.partHit instanceof IHitEffectsPart)
                if (((IHitEffectsPart) hit.partHit).addDestroyEffects(AdvancedEffectRenderer.getInstance(effectRenderer))) return true;

            String path = hit.partHit.getModelPath();
            IBlockState state = hit.partHit.getExtendedState(MultipartRegistry.getDefaultState(hit.partHit).func_177621_b());
            IBakedModel model = path == null ? null : Minecraft.func_71410_x().func_175602_ab().func_175023_a()
                    .func_178126_b()
                    .func_174953_a(new ModelResourceLocation(path, MultipartStateMapper.instance.func_178131_a(state.func_177228_b())));
            if (model != null) {
                model = model instanceof ISmartMultipartModel ? ((ISmartMultipartModel) model).handlePartState(hit.partHit
                        .getExtendedState(MultipartRegistry.getDefaultState(hit.partHit).func_177621_b())) : model;
                if (model != null) {
                    TextureAtlasSprite icon = model.func_177554_e();
                    if (icon != null) {
                        AdvancedEffectRenderer.getInstance(effectRenderer).addBlockDestroyEffects(pos, icon);
                        return true;
                    }
                }
            }
            return true;
        }
        return addDestroyEffectsDefault(world, pos, effectRenderer);
    }

    @SideOnly(Side.CLIENT)
    public boolean addDestroyEffectsDefault(World world, BlockPos pos, EffectRenderer effectRenderer) {

        return super.addDestroyEffects(world, pos, effectRenderer);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public final boolean addHitEffects(World world, MovingObjectPosition target, EffectRenderer effectRenderer) {

        PartMOP hit = target instanceof PartMOP ? (PartMOP) target : null;
        if (hit != null) {
            if (hit.partHit instanceof IHitEffectsPart)
                if (((IHitEffectsPart) hit.partHit).addHitEffects(hit, AdvancedEffectRenderer.getInstance(effectRenderer))) return true;

            String path = hit.partHit.getModelPath();
            IBlockState state = hit.partHit.getExtendedState(MultipartRegistry.getDefaultState(hit.partHit).func_177621_b());
            IBakedModel model = path == null ? null : Minecraft.func_71410_x().func_175602_ab().func_175023_a()
                    .func_178126_b()
                    .func_174953_a(new ModelResourceLocation(path, MultipartStateMapper.instance.func_178131_a(state.func_177228_b())));
            if (model != null) {
                model = model instanceof ISmartMultipartModel ? ((ISmartMultipartModel) model).handlePartState(hit.partHit
                        .getExtendedState(MultipartRegistry.getDefaultState(hit.partHit).func_177621_b())) : model;
                if (model != null) {
                    TextureAtlasSprite icon = model.func_177554_e();
                    if (icon != null) {
                        AdvancedEffectRenderer.getInstance(effectRenderer).addBlockHitEffects(
                                target.func_178782_a(),
                                hit,
                                world.func_180495_p(target.func_178782_a()).func_177230_c().func_180646_a(world, target.func_178782_a())
                                        .func_72317_d(-target.func_178782_a().func_177958_n(), -target.func_178782_a().func_177956_o(), -target.func_178782_a().func_177952_p()),
                                icon);
                        return true;
                    }
                }
            }
            return true;
        }
        return addHitEffectsDefault(world, target, effectRenderer);
    }

    @SideOnly(Side.CLIENT)
    public boolean addHitEffectsDefault(World world, MovingObjectPosition target, EffectRenderer effectRenderer) {

        return super.addHitEffects(world, target, effectRenderer);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean addLandingEffects(WorldServer worldObj, BlockPos blockPosition, IBlockState iblockstate, EntityLivingBase entity,
            int numberOfParticles) {

        return true;
    }

    private PartMOP reTrace(World world, BlockPos pos, EntityPlayer player) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        if (tile == null) return null;
        Vec3 start = RayTraceUtils.getStart(player);
        Vec3 end = RayTraceUtils.getEnd(player);
        RayTraceResultPart result = tile.getMicroblockContainer().getPartContainer().collisionRayTrace(start, end);
        return result == null ? null : result.hit;
    }

    private MovingObjectPosition reTraceBlock(World world, BlockPos pos, EntityPlayer player) {

        Vec3 start = RayTraceUtils.getStart(player);
        Vec3 end = RayTraceUtils.getEnd(player);
        return collisionRayTraceDefault(world, pos, start, end);
    }

    private MovingObjectPosition reTraceAll(World world, BlockPos pos, EntityPlayer player) {

        Vec3 start = RayTraceUtils.getStart(player);
        PartMOP partMOP = reTrace(world, pos, player);
        MovingObjectPosition blockMOP = reTraceBlock(world, pos, player);
        if (partMOP == null && blockMOP == null) return null;
        if (partMOP == null && blockMOP != null) return blockMOP;
        if (partMOP != null && blockMOP == null) return partMOP;
        if (partMOP.field_72307_f.func_72436_e(start) <= blockMOP.field_72307_f.func_72436_e(start)) return partMOP;
        return blockMOP;
    }

    @Override
    public int func_149645_b() {

        return 3;
    }

    @Override
    public boolean func_149637_q() {

        return false;
    }

    @Override
    public boolean func_149662_c() {

        return false;
    }

    @Override
    public boolean func_149686_d() {

        return false;
    }

    @Override
    public boolean func_149730_j() {

        return false;
    }

    @Override
    public IExtendedBlockState getExtendedState(IBlockState state, IBlockAccess world, BlockPos pos) {

        IMicroblockTile tile = getMicroblockTile(world, pos);
        return ((IExtendedBlockState) state).withProperty(BlockMultipart.properties[0], tile != null ? tile.getMicroblockContainer()
                .getPartContainer().getExtendedStates(world, pos) : new ArrayList<PartState>());
    }

    @Override
    public int func_176201_c(IBlockState state) {

        return 0;
    }

    @Override
    protected BlockState func_180661_e() {

        return new ExtendedBlockState(this, new IProperty[0], BlockMultipart.properties);
    }

    @Override
    public final boolean canRenderInLayer(EnumWorldBlockLayer layer) {

        return true;
    }

    public boolean canRenderInLayerDefault(EnumWorldBlockLayer layer) {

        return super.canRenderInLayer(layer);
    }

    @SubscribeEvent(priority = EventPriority.HIGH)
    @SideOnly(Side.CLIENT)
    public final void onDrawBlockHighlight(DrawBlockHighlightEvent event) {

        PartMOP hit = event.target instanceof PartMOP ? (PartMOP) event.target : null;
        if (hit != null) {
            GlStateManager.func_179094_E();

            BlockPos pos = hit.func_178782_a();
            EntityPlayer player = event.player;
            float partialTicks = event.partialTicks;
            double x = pos.func_177958_n() - (player.field_70142_S + (player.field_70165_t - player.field_70142_S) * partialTicks);
            double y = pos.func_177956_o() - (player.field_70137_T + (player.field_70163_u - player.field_70137_T) * partialTicks);
            double z = pos.func_177952_p() - (player.field_70136_U + (player.field_70161_v - player.field_70136_U) * partialTicks);
            GlStateManager.func_179137_b(x, y, z);

            if (hit.partHit instanceof ICustomHighlightPart
                    && ((ICustomHighlightPart) hit.partHit).drawHighlight(hit, event.player, event.currentItem, event.partialTicks))
                event.setCanceled(true);

            GlStateManager.func_179121_F();
        } else {
            onDrawBlockHighlightDefault(event);
        }
    }

    @SideOnly(Side.CLIENT)
    public void onDrawBlockHighlightDefault(DrawBlockHighlightEvent event) {

    }

}
