package mcmultipart.item;

import java.util.HashMap;
import java.util.Map;

import mcmultipart.multipart.IMultipart;
import mcmultipart.multipart.MultipartHelper;
import mcmultipart.network.MessageWrappedPartPlacement;
import mcmultipart.raytrace.RayTraceUtils;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import com.google.common.base.Predicate;

public class PartPlacementWrapper {

    private static final Map<String, PartPlacementWrapper> wrappers = new HashMap<String, PartPlacementWrapper>();

    public static PartPlacementWrapper getWrapper(String handler) {

        return wrappers.get(handler);
    }

    protected final Predicate<ItemStack> match;
    protected final IItemMultipartFactory factory;
    private String identifier;

    public PartPlacementWrapper(Predicate<ItemStack> match, IItemMultipartFactory factory) {

        this.match = match;
        this.factory = factory;
    }

    public PartPlacementWrapper(final ItemStack match, IItemMultipartFactory factory) {

        this(new Predicate<ItemStack>() {

            @Override
            public boolean apply(ItemStack input) {

                return input.func_77969_a(match);
            }
        }, factory);
    }

    public void register(String identifier) {

        wrappers.put(this.identifier = identifier, this);
        MinecraftForge.EVENT_BUS.register(this);
    }

    protected boolean place(World world, BlockPos pos, EnumFacing side, Vec3 hit, ItemStack stack, EntityPlayer player) {

        if (!player.func_175151_a(pos, side, stack)) return false;

        IMultipart mb = factory.createPart(world, pos, side, hit, stack, player);

        if (MultipartHelper.canAddPart(world, pos, mb)) {
            if (!world.field_72995_K) MultipartHelper.addPart(world, pos, mb);
            if (!player.field_71075_bZ.field_75098_d) consumeItem(stack);

            playPlacementSound(world, pos, stack);

            return true;
        }
        return false;
    }

    protected boolean placeDefault(World world, BlockPos pos, EnumFacing side, Vec3 hit, ItemStack stack, EntityPlayer player) {

        IBlockState iblockstate = world.func_180495_p(pos);
        Block block = iblockstate.func_177230_c();

        Block placedBlock = Block.func_149634_a(stack.func_77973_b());
        if (placedBlock == null)
            throw new IllegalStateException("For non-ItemBlocks you need to write your own default placement handler!");

        if (!block.func_176200_f(world, pos)) return false;

        if (stack.field_77994_a == 0) {
            return false;
        } else if (!player.func_175151_a(pos, side, stack)) {
            return false;
        } else if (world.func_175716_a(placedBlock, pos, false, side, (Entity) null, stack)) {
            if (world.field_72995_K) return true;

            int i = stack.func_77973_b().func_77647_b(stack.func_77960_j());
            IBlockState iblockstate1 = placedBlock.func_180642_a(world, pos, side, (float) hit.field_72450_a, (float) hit.field_72448_b,
                    (float) hit.field_72449_c, i, player);

            if (((ItemBlock) stack.func_77973_b()).placeBlockAt(stack, player, world, pos, side, (float) hit.field_72450_a, (float) hit.field_72448_b,
                    (float) hit.field_72449_c, iblockstate1)) {
                playPlacementSound(world, pos, stack);
                if (!player.field_71075_bZ.field_75098_d) consumeItem(stack);
            }

            return true;
        } else {
            return false;
        }
    }

    protected void consumeItem(ItemStack stack) {

        stack.field_77994_a--;
    }

    protected void playPlacementSound(World world, BlockPos pos, ItemStack stack) {

        Block placedBlock = Block.func_149634_a(stack.func_77973_b());
        if (placedBlock != null)
            world.func_72908_a(pos.func_177958_n() + 0.5F, pos.func_177956_o() + 0.5F, pos.func_177952_p() + 0.5F, placedBlock.field_149762_H.func_150496_b(),
                    (placedBlock.field_149762_H.func_150497_c() + 1.0F) / 2.0F, placedBlock.field_149762_H.func_150494_d() * 0.8F);;

        Block.SoundType sound = getPlacementSound(stack);
        if (sound != null)
            world.func_72908_a(pos.func_177958_n() + 0.5, pos.func_177956_o() + 0.5, pos.func_177952_p() + 0.5, sound.func_150496_b(), sound.func_150497_c(),
                    sound.func_150494_d());
    }

    protected Block.SoundType getPlacementSound(ItemStack stack) {

        return Block.field_149778_k;
    }

    protected boolean isValidPlacement(World world, BlockPos pos, EnumFacing side) {

        return true;
    }

    @SubscribeEvent
    public void onPlayerInteract(PlayerInteractEvent event) {

        ItemStack stack = event.entityPlayer.func_71045_bC();
        if (stack == null || !match.apply(stack) || event.action != PlayerInteractEvent.Action.RIGHT_CLICK_BLOCK) return;

        event.setCanceled(true);

        World world = event.world;
        BlockPos pos = event.pos;
        EnumFacing side = event.face;
        EntityPlayer player = event.entityPlayer;
        MovingObjectPosition mop = world.func_72933_a(RayTraceUtils.getStart(player), RayTraceUtils.getEnd(player));
        Vec3 hit = mop.field_72307_f.func_178788_d(new Vec3(mop.func_178782_a()));

        if (doPlace(world, pos, side, hit, stack, player)) {
            player.func_71038_i();
            if (world.field_72995_K) new MessageWrappedPartPlacement(identifier).send();
        }
    }

    public boolean doPlace(World world, BlockPos pos, EnumFacing side, Vec3 hit, ItemStack stack, EntityPlayer player) {

        if (doPlaceAt(world, pos, side, hit, stack, player)) return true;
        pos = pos.func_177972_a(side);
        return doPlaceAt(world, pos, side, hit, stack, player);
    }

    private boolean doPlaceAt(World world, BlockPos pos, EnumFacing side, Vec3 hit, ItemStack stack, EntityPlayer player) {

        Block block = world.func_180495_p(pos).func_177230_c();
        if (block.func_176200_f(world, pos) && placeDefault(world, pos, side, hit, stack, player)) return true;
        return isValidPlacement(world, pos, side) && place(world, pos, side, hit, stack, player);
    }

}
