package mcmultipart.util;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.vecmath.Vector2d;

import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Vec3;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class TransformationHelper {

    private static final FloatBuffer[] glMatrices = new FloatBuffer[6];

    private static FloatBuffer buf() {

        return ByteBuffer.allocateDirect(4 * (4 * 4)).order(ByteOrder.nativeOrder()).asFloatBuffer();
    }

    static {
        glMatrices[0] = (FloatBuffer) buf().put(new float[] { 1, 0, 0, 0, 0, -1, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1 }).flip();
        glMatrices[1] = (FloatBuffer) buf().put(new float[] { 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 }).flip();
        glMatrices[2] = (FloatBuffer) buf().put(new float[] { 1, 0, 0, 0, 0, 0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 1 }).flip();
        glMatrices[3] = (FloatBuffer) buf().put(new float[] { 1, 0, 0, 0, 0, 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 1 }).flip();
        glMatrices[4] = (FloatBuffer) buf().put(new float[] { 0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 }).flip();
        glMatrices[5] = (FloatBuffer) buf().put(new float[] { 0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1 }).flip();
    }

    @SideOnly(Side.CLIENT)
    public static void glRotate(EnumFacing face) {

        GlStateManager.func_179110_a(glMatrices[face.ordinal()]);
    }

    public static AxisAlignedBB rotate(AxisAlignedBB aabb, EnumFacing side) {

        Vec3 v1 = rotate(new Vec3(aabb.field_72340_a, aabb.field_72338_b, aabb.field_72339_c), side);
        Vec3 v2 = rotate(new Vec3(aabb.field_72336_d, aabb.field_72337_e, aabb.field_72334_f), side);
        return new AxisAlignedBB(v1.field_72450_a, v1.field_72448_b, v1.field_72449_c, v2.field_72450_a, v2.field_72448_b, v2.field_72449_c);
    }

    public static Vec3 rotate(Vec3 vec, EnumFacing side) {

        switch (side) {
        case DOWN:
            return new Vec3(vec.field_72450_a, vec.field_72448_b, vec.field_72449_c);
        case UP:
            return new Vec3(vec.field_72450_a, -vec.field_72448_b, -vec.field_72449_c);
        case NORTH:
            return new Vec3(vec.field_72450_a, -vec.field_72449_c, vec.field_72448_b);
        case SOUTH:
            return new Vec3(vec.field_72450_a, vec.field_72449_c, -vec.field_72448_b);
        case WEST:
            return new Vec3(vec.field_72448_b, -vec.field_72450_a, vec.field_72449_c);
        case EAST:
            return new Vec3(-vec.field_72448_b, vec.field_72450_a, vec.field_72449_c);
        }
        return null;
    }

    public static AxisAlignedBB revRotate(AxisAlignedBB aabb, EnumFacing side) {

        Vec3 v1 = revRotate(new Vec3(aabb.field_72340_a, aabb.field_72338_b, aabb.field_72339_c), side);
        Vec3 v2 = revRotate(new Vec3(aabb.field_72336_d, aabb.field_72337_e, aabb.field_72334_f), side);
        return new AxisAlignedBB(v1.field_72450_a, v1.field_72448_b, v1.field_72449_c, v2.field_72450_a, v2.field_72448_b, v2.field_72449_c);
    }

    public static Vec3 revRotate(Vec3 vec, EnumFacing side) {

        switch (side) {
        case DOWN:
            return new Vec3(vec.field_72450_a, vec.field_72448_b, vec.field_72449_c);
        case UP:
            return new Vec3(vec.field_72450_a, -vec.field_72448_b, -vec.field_72449_c);
        case NORTH:
            return new Vec3(vec.field_72450_a, vec.field_72449_c, -vec.field_72448_b);
        case SOUTH:
            return new Vec3(vec.field_72450_a, -vec.field_72449_c, vec.field_72448_b);
        case WEST:
            return new Vec3(-vec.field_72448_b, vec.field_72450_a, vec.field_72449_c);
        case EAST:
            return new Vec3(vec.field_72448_b, -vec.field_72450_a, vec.field_72449_c);
        }
        return null;
    }

    public static Vector2d project(Vec3 vec, EnumFacing sideHit) {

        double x = vec.field_72450_a, z = vec.field_72449_c;

        if (sideHit == EnumFacing.DOWN || sideHit == EnumFacing.UP) {
            double a = x;
            x = z;
            z = a;
        } else if (sideHit == EnumFacing.NORTH || sideHit == EnumFacing.SOUTH) {
            z = vec.field_72448_b;
        } else if (sideHit == EnumFacing.WEST || sideHit == EnumFacing.EAST) {
            x = vec.field_72448_b;
        }

        if (x < 0) x = 1 + x;
        if (z < 0) z = 1 + z;

        return new Vector2d(x, z);
    }

}
